// CGraphView.cpp : implementation file
//

#include "stdafx.h"
#include "ModelApp.h"
#include "ModelDoc.h"
#include "CGraphView.h"
#include "wgraphview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGraphView

IMPLEMENT_DYNCREATE(CGraphView, CScrollView)

CGraphView::CGraphView()
{
  wgraphview = NULL;

  // Remainder of initialisation done in Init().
}

CGraphView::~CGraphView()
{
  // Most deletion is done in OnDestroy().
}


BEGIN_MESSAGE_MAP(CGraphView, CScrollView)
	//{{AFX_MSG_MAP(CGraphView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphView drawing

void CGraphView::OnDraw(CDC* pDC)
{
  int xsize, ysize, i, j;
  CRect rectClip;
  CPen currentPen, *oldPen;
  CBrush currentBrush, *oldBrush;
  CPoint box[4], arrowpt[2];
  Mirage::nodeinfo *nextnode;
  Mirage::linkinfo *nextlink;
  double arrowxdir, arrowydir, dirmag;
  UINT oldTextAlign, oldBkMode;
  COLORREF oldTextColor;
  CFont newFont, *oldFont;

  CDocument* pDoc = GetDocument();

  // Set the initial size of the graphview.
  wgraphview->get_size(&xsize, &ysize);
  SetScrollSizes(MM_TEXT, CSize(xsize, ysize));

  //Note: CScrollView::OnPaint() will have already adjusted the
  //viewpoint origin before calling OnDraw(), to reflect the
  //currently scrolled position.

  // Draw all the links.
  for (i = 0, nextlink = wgraphview->linkarray; i < wgraphview->num_links; i++, nextlink++) {
    if (nextlink->is_ghost)
      currentPen.CreatePen(PS_SOLID, 1, RGB(0, 0, 255));
    else
      currentPen.CreatePen(PS_SOLID, 1, RGB(0, 0, 0));

    oldPen = pDC->SelectObject(&currentPen);

    // Move to the start of the link.
    pDC->MoveTo(CPoint(nextlink->segpos[0][0], nextlink->segpos[0][1]));

    // Draw each line in the link.
    for (j = 1; j < nextlink->num_segs; j++)
      pDC->LineTo(CPoint(nextlink->segpos[j][0], nextlink->segpos[j][1]));

    // If we have to, draw an arrow on the end of the link.
    if (nextlink->has_arrow) {
      arrowxdir = nextlink->segpos[nextlink->num_segs - 1][0] - nextlink->segpos[nextlink->num_segs - 2][0];
      arrowydir = nextlink->segpos[nextlink->num_segs - 1][1] - nextlink->segpos[nextlink->num_segs - 2][1];
      dirmag = sqrt(arrowxdir * arrowxdir + arrowydir * arrowydir);

      arrowpt[0].x = (int) (nextlink->segpos[nextlink->num_segs - 1][0] - arrowxdir * 9.0 / dirmag - arrowydir * 3.0 / dirmag);
      arrowpt[0].y = (int) (nextlink->segpos[nextlink->num_segs - 1][1] - arrowydir * 9.0 / dirmag + arrowxdir * 3.0 / dirmag);
      arrowpt[1].x = (int) (nextlink->segpos[nextlink->num_segs - 1][0] - arrowxdir * 9.0 / dirmag + arrowydir * 3.0 / dirmag);
      arrowpt[1].y = (int) (nextlink->segpos[nextlink->num_segs - 1][1] - arrowydir * 9.0 / dirmag - arrowxdir * 3.0 / dirmag);

      pDC->LineTo(arrowpt[0]);
      pDC->MoveTo(CPoint(nextlink->segpos[nextlink->num_segs - 1][0], nextlink->segpos[nextlink->num_segs - 1][1]));
      pDC->LineTo(arrowpt[1]);
    }

    pDC->SelectObject(oldPen);
    currentPen.DeleteObject();
  }

  // Draw all the nodes.
  for (i = 0, nextnode = wgraphview->nodearray; i < wgraphview->num_nodes; i++, nextnode++) {
    currentPen.CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
    if (nextnode->is_selected)
      currentBrush.CreateSolidBrush(RGB(128, 128, 255));
    else
      currentBrush.CreateSolidBrush(RGB(255, 255, 255));
    oldPen = pDC->SelectObject(&currentPen);
    oldBrush = pDC->SelectObject(&currentBrush);

    // Construct the node's corners.
    box[0].x = nextnode->pos[0] - nextnode->size / 2;
    box[0].y = nextnode->pos[1] - nextnode->size / 2;
    box[1].x = nextnode->pos[0] + nextnode->size / 2;
    box[1].y = nextnode->pos[1] - nextnode->size / 2;
    box[2].x = nextnode->pos[0] + nextnode->size / 2;
    box[2].y = nextnode->pos[1] + nextnode->size / 2;
    box[3].x = nextnode->pos[0] - nextnode->size / 2;
    box[3].y = nextnode->pos[1] + nextnode->size / 2;

    // Draw the polygon.
    pDC->Polygon(box, 4);

    // If there is any internal text to the node, draw it.
    if (strcmp(nextnode->internaltext, "") != 0) {
      oldTextAlign = pDC->SetTextAlign(TA_CENTER | TA_BASELINE);
      oldBkMode = pDC->SetBkMode(TRANSPARENT);
      oldTextColor = pDC->SetTextColor(RGB(0, 0, 0));

      newFont.CreatePointFont(nextnode->size * 7, "TimesRoman");
      oldFont = pDC->SelectObject(&newFont);


      pDC->TextOut(nextnode->pos[0], nextnode->pos[1] + nextnode->size/2 - 2,
                   nextnode->internaltext, strlen(nextnode->internaltext));

      pDC->SelectObject(oldFont);
      newFont.DeleteObject();
      pDC->SetTextColor(oldTextColor);
      pDC->SetTextAlign(oldTextAlign);
      pDC->SetBkMode(oldBkMode);
    }

    pDC->SelectObject(oldPen);
    pDC->SelectObject(oldBrush);
    currentPen.DeleteObject();
    currentBrush.DeleteObject();
  }

  // Set the current selection state of the ModelDoc.. if nodes are selected,
  // enable DELETE and LINK buttons.
  GetDocument()->OnSetSelectViewState();
}

/////////////////////////////////////////////////////////////////////////////
// CGraphView diagnostics

#ifdef _DEBUG
void CGraphView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CGraphView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CModelDoc* CGraphView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CModelDoc)));
	return (CModelDoc*)m_pDocument;
}
#endif // _DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGraphView message handlers

int CGraphView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  int xsize, ysize;

	if (CScrollView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create the Mirage WObjectGraphView class that's attached to this View.
  wgraphview = new Mirage::WObjectGraphView(GetDocument()->modeller, this);

  // Set the initial size of the graphview.
  wgraphview->get_size(&xsize, &ysize);
  SetScrollSizes(MM_TEXT, CSize(xsize, ysize));

  return 0;
}

void CGraphView::OnDestroy() 
{
	CScrollView::OnDestroy();
	
  // Delete the WObjectGraphView.
  if (wgraphview != NULL) {
    delete wgraphview;
    wgraphview = NULL;
  }
}

void CGraphView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// We've registered a left-button press in the graph view.

 	// CScrollView changes the viewport origin and mapping
	// mode. Thus, we must convert the mouse-point from
	// device coordinates to logical coordinates relative
	// to the document.
	CClientDC dc(this);
	OnPrepareDC(&dc); // Set up origin
	dc.DPtoLP(&point); // Move point to logical coords.

  wgraphview->selection_click(point.x, point.y);
}

