// GLView.cpp : implementation of the GLView class
//

#include "stdafx.h"

#include "ModelApp.h"
#include "ModelDoc.h"
#include "GLView.h"
#include "wglview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


unsigned char threeto8[8] = 
{
	0, 0111>>1, 0222>>1, 0333>>1, 0444>>1, 0555>>1, 0666>>1, 0377
};

unsigned char twoto8[4] = 
{
	0, 0x55, 0xaa, 0xff
};

unsigned char oneto8[2] = 
{
	0, 255
};

static int defaultOverride[13] = 
{
	0, 3, 24, 27, 64, 67, 88, 173, 181, 236, 247, 164, 91
};

static PALETTEENTRY defaultPalEntry[20] = 
{
	{ 0,   0,   0,    0 },
	{ 0x80,0,   0,    0 },
	{ 0,   0x80,0,    0 },
	{ 0x80,0x80,0,    0 },
	{ 0,   0,   0x80, 0 },
	{ 0x80,0,   0x80, 0 },
	{ 0,   0x80,0x80, 0 },
	{ 0xC0,0xC0,0xC0, 0 },

	{ 192, 220, 192,  0 },
	{ 166, 202, 240,  0 },
	{ 255, 251, 240,  0 },
	{ 160, 160, 164,  0 },

	{ 0x80,0x80,0x80, 0 },
	{ 0xFF,0,   0,    0 },
	{ 0,   0xFF,0,    0 },
	{ 0xFF,0xFF,0,    0 },
	{ 0,   0,   0xFF, 0 },
	{ 0xFF,0,   0xFF, 0 },
	{ 0,   0xFF,0xFF, 0 },
	{ 0xFF,0xFF,0xFF, 0 }
};


/////////////////////////////////////////////////////////////////////////////
// GLView

IMPLEMENT_DYNCREATE(GLView, CView)

BEGIN_MESSAGE_MAP(GLView, CView)
	//{{AFX_MSG_MAP(GLView)
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_CREATE()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_DESTROY()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// GLView construction/destruction

GLView::GLView()
{
  m_pDC = NULL;
  modeller_view = NULL;
  bAPICreated = FALSE;

  // All initialisation is done in
  // Init(), called from OnCreate.
}

GLView::~GLView()
{
  // Most deletion is done in OnDestroy.
}

/////////////////////////////////////////////////////////////////////////////
// Main drawing

void GLView::OnDraw(CDC* pDC)
{
  static BOOL bBusy = FALSE;

  if(bBusy) 
		return;

  bBusy = TRUE;

  modeller_view->display();

  bBusy = FALSE;

}

BOOL GLView::OnEraseBkgnd(CDC* pDC) 
{	
	return true;
}

/////////////////////////////////////////////////////////////////////////////
// GLView diagnostics

#ifdef _DEBUG
void GLView::AssertValid() const
{
	CView::AssertValid();
}

void GLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CModelDoc* GLView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CModelDoc)));
	return (CModelDoc*)m_pDocument;
}
#endif //_DEBUG

////////////////////////////////////////////////////////////////////////////
// High-ish level view code

void GLView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
  if(cy > 0) {
    wglMakeCurrent(m_pDC->GetSafeHdc(), hGLRenderContext);

    modeller_view->resize(cx, cy);
  }
}

int GLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  HRESULT hr;

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  m_pDC = new CClientDC(this);
  ASSERT(m_pDC != NULL);

  // Call the rendering API initialisation.
  hr = InitAPI();
	if (FAILED(hr))
    return hr;

  bAPICreated = TRUE;

  // Set up the Modeller's view class (a subclass of View).
  modeller_view = CreateModellerView();

	return 0;
}

void GLView::windowspace_to_viewspace(CPoint windowPt, double *x, double *y)
{
  *x = windowPt.x - (modeller_view->view_width  / 2.0);
  *y = windowPt.y - (modeller_view->view_height / 2.0);

  *x /= (modeller_view->view_height / 2.0);
  *y /= (modeller_view->view_height / 2.0);
}

void GLView::OnLButtonDown(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	SetCapture();
  drag_leftbutton = TRUE;

  windowspace_to_viewspace(point, &view_x, &view_y);

  modeller_view->mouse_event(view_x, view_y,
                             1, Mirage::MOUSEEVENT_PRESS, MODIFIER_NONE |
                             ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                             ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));

  CView::OnLButtonDown(nFlags, point);
}

void GLView::OnMouseMove(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

  if (GetCapture() == this) {
    windowspace_to_viewspace(point, &view_x, &view_y);

    modeller_view->mouse_event(view_x, view_y,
                               (drag_leftbutton) ? 1 : 2,
                               Mirage::MOUSEEVENT_MOTION, MODIFIER_NONE |
                               ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                               ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));
  }
	
	CView::OnMouseMove(nFlags, point);
}

void GLView::OnLButtonUp(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	if (GetCapture()) {
		ReleaseCapture();

    windowspace_to_viewspace(point, &view_x, &view_y);

    modeller_view->mouse_event(view_x, view_y,
                               1, Mirage::MOUSEEVENT_RELEASE, MODIFIER_NONE |
                               ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                               ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));
  }
	CView::OnLButtonUp(nFlags, point);
}

void GLView::OnRButtonDown(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	SetCapture();
  drag_leftbutton = FALSE;

  windowspace_to_viewspace(point, &view_x, &view_y);

  modeller_view->mouse_event(view_x, view_y,
                             2, Mirage::MOUSEEVENT_PRESS, MODIFIER_NONE |
                             ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                             ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));

  CView::OnRButtonDown(nFlags, point);
}

void GLView::OnRButtonUp(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	if (GetCapture()) {
		ReleaseCapture();

    windowspace_to_viewspace(point, &view_x, &view_y);

    modeller_view->mouse_event(view_x, view_y,
                               2, Mirage::MOUSEEVENT_RELEASE, MODIFIER_NONE |
                               ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                               ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));
  }
	CView::OnRButtonUp(nFlags, point);
}

void GLView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
  wglMakeCurrent(m_pDC->GetSafeHdc(), hGLRenderContext);

	CView::OnPrepareDC(pDC, pInfo);
}

void GLView::OnDestroy() 
{
	if (m_pDC)         delete m_pDC;
  if (modeller_view) delete modeller_view;
  if (bAPICreated)   ReleaseAPI();

  m_pDC = NULL;
  bAPICreated = FALSE;
  modeller_view = NULL;

	CView::OnDestroy();
}

BOOL GLView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
  modeller_view->zoom_event(zDelta / 120);
    
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

/////////////////////////////////////////////////////////////////////////////
// GLView member functions

HRESULT GLView::InitAPI(void)
{
  HRESULT hr;

  hr = SetupPixelFormat();
  if (FAILED(hr))
    return hr;

  CreateRGBPalette();

  hGLRenderContext = wglCreateContext(m_pDC->GetSafeHdc());
  wglMakeCurrent(m_pDC->GetSafeHdc(), hGLRenderContext);
  
#if _DEBUG
	// TRACE the OpenGL Implementation Information to check on the setup
	const char* vendor = (const char*) glGetString( GL_VENDOR );
	const char* renderer = (const char*) glGetString( GL_RENDERER );
	const char* version = (const char*) glGetString( GL_VERSION );
	const char* extensions = (const char*) glGetString( GL_EXTENSIONS );
	int stackdepth;
	glGetIntegerv( GL_MAX_NAME_STACK_DEPTH, &stackdepth );
	TRACE0("OpenGL information:\n");
	TRACE1(" Vendor :	'%s'\n", vendor);
	TRACE1(" Renderer :	'%s'\n", renderer);
	TRACE1(" Version :	'%s'\n", version);
	TRACE1(" Extensions:'%s'\n", extensions);
	TRACE1(" Selection Name Stack Depth: %d\n", stackdepth);
	if (stackdepth == GL_INVALID_ENUM)
    TRACE(" Invalid stack depth enum\n");
	if (stackdepth == GL_INVALID_OPERATION)
    TRACE(" Invalid stack depth operation\n");
#endif

  return S_OK;
}

HRESULT GLView::SetupPixelFormat(void)
{
  static PIXELFORMATDESCRIPTOR pfd = 
	{
    sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
    1,                              // version number
    PFD_DRAW_TO_WINDOW |            // support window
      PFD_SUPPORT_OPENGL |          // support OpenGL
      PFD_DOUBLEBUFFER,             // double buffered
    PFD_TYPE_RGBA,                  // RGBA type
    24,                             // 24-bit color depth
    0, 0, 0, 0, 0, 0,               // color bits ignored
    0,                              // no alpha buffer
    0,                              // shift bit ignored
    0,                              // no accumulation buffer
    0, 0, 0, 0,                     // accum bits ignored
    32,                             // 32-bit z-buffer
    1,                              // 1-bit stencil buffer
    0,                              // no auxiliary buffer
    PFD_MAIN_PLANE,                 // main layer
    0,                              // reserved
    0, 0, 0                         // layer masks ignored
  };
  int pixelformat;

  if ((pixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0) {
    MessageBox("Cannot create a 24-bit OpenGL window with 32-bit Z-buffer, double buffering and a stencil buffer.");
    return E_FAIL;
  }

  if (SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE) {
    MessageBox("Cannot select PixelFormat that was found.");
    return E_FAIL;
  }

  return S_OK;
}

unsigned char GLView::ComponentFromIndex(int i, UINT nbits, UINT shift)
{
  unsigned char val = (unsigned char) (i >> shift);

  switch (nbits) {

  case 1:
    val &= 0x1;
    return oneto8[val];
  case 2:
    val &= 0x3;
    return twoto8[val];
  case 3:
    val &= 0x7;
    return threeto8[val];

  default:
    return 0;
  }
}

void GLView::CreateRGBPalette()
{
  PIXELFORMATDESCRIPTOR pfd;
  LOGPALETTE *pPal;
  int n, i;
  CPalette cPalette;
 
  n = ::GetPixelFormat(m_pDC->GetSafeHdc());
  ::DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(pfd), &pfd);

  if (pfd.dwFlags & PFD_NEED_PALETTE) {
    n = 1 << pfd.cColorBits;
    pPal = (PLOGPALETTE) new char[sizeof(LOGPALETTE) + n * sizeof(PALETTEENTRY)];

    ASSERT(pPal != NULL);

    pPal->palVersion = 0x300;
    pPal->palNumEntries = n;
    for (i=0; i<n; i++) {
      pPal->palPalEntry[i].peRed = ComponentFromIndex(i, pfd.cRedBits, pfd.cRedShift);
      pPal->palPalEntry[i].peGreen = ComponentFromIndex(i, pfd.cGreenBits, pfd.cGreenShift);
      pPal->palPalEntry[i].peBlue = ComponentFromIndex(i, pfd.cBlueBits, pfd.cBlueShift);
      pPal->palPalEntry[i].peFlags = 0;
    }

    /* fix up the palette to include the default GDI palette */
    if ((pfd.cColorBits == 8)                           &&
        (pfd.cRedBits   == 3) && (pfd.cRedShift   == 0) &&
        (pfd.cGreenBits == 3) && (pfd.cGreenShift == 3) &&
        (pfd.cBlueBits  == 2) && (pfd.cBlueShift  == 6)) {
			for (i = 1 ; i <= 12 ; i++)
        pPal->palPalEntry[defaultOverride[i]] = defaultPalEntry[i];
    }

    cPalette.CreatePalette(pPal);
    delete pPal;

    m_pDC->SelectPalette(&cPalette, FALSE);
    m_pDC->RealizePalette();
  }
}

// Create the Modeller's view class to interface to.
Mirage::WGLView *GLView::CreateModellerView(void)
{
  return new Mirage::WGLView(GetDocument()->modeller, this);
}

// Release the 3D API.
void GLView::ReleaseAPI(void)
{
  wglMakeCurrent(NULL, NULL);
  if (hGLRenderContext) wglDeleteContext(hGLRenderContext);
  hGLRenderContext = NULL;
}
