// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "ModelApp.h"
#include "ModelDoc.h"
#include "GLView.h"
#include "CGraphView.h"
#include "CTransView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_PRIMITIVE, OnPrimitive)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

// Used in handling WM_NOTIFY
#define lpnm   ((LPNMHDR)lParam)
#define lpnmTB ((LPNMTOOLBAR)lParam)

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

  CToolBarCtrl &tbctrl = m_wndToolBar.GetToolBarCtrl();
  tbctrl.SetExtendedStyle(tbctrl.GetExtendedStyle() |
                                 TBSTYLE_EX_DRAWDDARROWS);

  // Set the ID_PRIMITIVE button to have a dropdown menu.
  TBBUTTONINFO buttoninfo;
  buttoninfo.cbSize = sizeof(buttoninfo);
  buttoninfo.dwMask = TBIF_STYLE;
  tbctrl.GetButtonInfo(ID_PRIMITIVE, &buttoninfo);
  buttoninfo.fsStyle |= TBSTYLE_DROPDOWN;
  tbctrl.SetButtonInfo(ID_PRIMITIVE, &buttoninfo);

	return 0;
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT /*lpcs*/,
	CCreateContext* pContext)
{
  // OnCreateClient must create the views that are inside the main frame.
  //
  // In the case of the GLCSG modeller, this is one model view
  // (CGLView or whatever, passed in via m_pNewViewClass),
  // a graphview (CGraphView) and a transformation view (CTransView).

  // Create the splitter that divides the view in two.
  if (!m_horizSplitter.CreateStatic(this, 1, 2)) {
    TRACE0("CMainFrame::OnCreateClient can't CreateStatic m_horizSplitter\n");
    return FALSE;
  }

  m_horizSplitter.SetColumnInfo(0, 300, 5);
  m_horizSplitter.SetColumnInfo(1, 300, 5);
  
  
  // Create the modeller-view in the left-hand pane.
  if (!m_horizSplitter.CreateView(0, 0,
    pContext->m_pNewViewClass, CSize(300, 300), pContext)) {
    TRACE0("CMainFrame::OnCreateClient can't CreateView left pane\n");
    return FALSE;
  }

  // Create a vertical splitter for the right-hand
  // (utility) pane.
  if (!m_utilityviewSplitter.CreateStatic(&m_horizSplitter,                   // Child of the horizontal splitter
                                          2, 1,                               // 2 rows, 1 column
                                          WS_CHILD | WS_VISIBLE | WS_BORDER,  // window style
                                          m_horizSplitter.IdFromRowCol(0, 1)  // place ourselves in row 0, col 1 of parent
                                          )) { 
    TRACE0("CMainFrame::OnCreateClient can't CreateStatic m_utilityviewSplitter\n");
    return FALSE;
  }

  m_utilityviewSplitter.SetRowInfo(0, 150, 5);
  m_utilityviewSplitter.SetRowInfo(1, 150, 5);
  
  // Create the graph view.
  if (!m_utilityviewSplitter.CreateView(0, 0,
                                        RUNTIME_CLASS(CGraphView),
                                        CSize(300, 145), pContext)) {
    TRACE0("CMainFrame::OnCreateClient can't CreateView CGraphView\n");
    return FALSE;
  }

  // Create the transformation view.
  if (!m_utilityviewSplitter.CreateView(1, 0,
                                        RUNTIME_CLASS(CTransView),
                                        CSize(300, 150), pContext)) {
    TRACE0("CMainFrame::OnCreateClient can't CreateView CTransView\n");
    return FALSE;
  }

  // Construction successful.
  return TRUE;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
  cs.cx = 600;
  cs.cy = 300;

	return CFrameWnd::PreCreateWindow(cs);
}

BOOL CMainFrame::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult) 
{
  switch(lpnm->code) {
  case TBN_DROPDOWN:
    // The primitives menu has been dropped down.
    // This is the same as a click on the primitive button.
    OnPrimitive();
    
    return FALSE; //indicates the TBN_DROPDOWN
                  //notification was handled.
  }

  return CFrameWnd::OnNotify(wParam, lParam, pResult);
}

// ID_PRIMITIVE has been sent. Pop-up the primitives menu.
void CMainFrame::OnPrimitive(void) 
{
  CMenu primmenu;
  VERIFY(primmenu.LoadMenu(IDR_PRIMITIVE_MENU));

  CMenu *pPopup = primmenu.GetSubMenu(0);
  assert(pPopup != NULL);

  // Find the location of the button.
  CRect buttonRect;
  m_wndToolBar.GetToolBarCtrl().GetRect(ID_PRIMITIVE,
                                          &buttonRect);
  ClientToScreen(&buttonRect);

  // Pop up the menu.
  pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON,
                         buttonRect.left, buttonRect.bottom,
                         this);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

