// ModelView.cpp : implementation of the ModelView class
//

#include "stdafx.h"

#include "ModelApp.h"
#include "ModelDoc.h"
#include "ModelView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




/////////////////////////////////////////////////////////////////////////////
// ModelView

IMPLEMENT_DYNCREATE(ModelView, CView)

BEGIN_MESSAGE_MAP(ModelView, CView)
	//{{AFX_MSG_MAP(ModelView)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ModelView construction/destruction

ModelView::ModelView()
{
  // Remainder of initialisation is done in
  // Init(), called from OnCreate.
}

ModelView::~ModelView()
{
  // Most deletion is done in OnDestroy.
}

/////////////////////////////////////////////////////////////////////////////
// ModelView drawing

void ModelView::OnDraw(CDC* pDC)
{
  static BOOL bBusy = FALSE;

  if(bBusy) 
		return;

  bBusy = TRUE;

  modeller_view->display();

  bBusy = FALSE;

}

/////////////////////////////////////////////////////////////////////////////
// ModelView diagnostics

#ifdef _DEBUG
void ModelView::AssertValid() const
{
	CView::AssertValid();
}

void ModelView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// ModelView message handlers

void ModelView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	if(cy > 0)
    modeller_view->resize(cx, cy);
}

int ModelView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  HRESULT hr;

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
  m_pDC = new CClientDC(this);
  ASSERT(m_pDC != NULL);

  // Call the subclass's API initialisation.
  hr = InitAPI();
	if (FAILED(hr))
    return hr;

  bAPICreated = TRUE;

  // Call the subclass to set up the Modeller's view class.
  modeller_view = CreateModellerView();

	return 0;
}

void ModelView::windowspace_to_viewspace(CPoint windowPt, double *x, double *y)
{
  *x = windowPt.x - (modeller_view->view_width  / 2.0);
  *y = windowPt.y - (modeller_view->view_height / 2.0);

  *x /= (modeller_view->view_height / 2.0);
  *y /= (modeller_view->view_height / 2.0);
}

void ModelView::OnLButtonDown(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	SetCapture();
  drag_leftbutton = TRUE;

  windowspace_to_viewspace(point, &view_x, &view_y);

  modeller_view->mouse_event(view_x, view_y,
                             1, Mirage::MOUSEEVENT_PRESS, MODIFIER_NONE |
                             ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                             ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));

  CView::OnLButtonDown(nFlags, point);
}

void ModelView::OnMouseMove(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

  if (GetCapture() == this) {
    windowspace_to_viewspace(point, &view_x, &view_y);

    modeller_view->mouse_event(view_x, view_y,
                               (drag_leftbutton) ? 1 : 2,
                               Mirage::MOUSEEVENT_MOTION, MODIFIER_NONE |
                               ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                               ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));
  }
	
	CView::OnMouseMove(nFlags, point);
}

void ModelView::OnLButtonUp(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	if (GetCapture()) {
		ReleaseCapture();

    windowspace_to_viewspace(point, &view_x, &view_y);

    modeller_view->mouse_event(view_x, view_y,
                               1, Mirage::MOUSEEVENT_RELEASE, MODIFIER_NONE |
                               ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                               ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));
  }
	CView::OnLButtonUp(nFlags, point);
}

void ModelView::OnRButtonDown(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	SetCapture();
  drag_leftbutton = FALSE;

  windowspace_to_viewspace(point, &view_x, &view_y);

  modeller_view->mouse_event(view_x, view_y,
                             2, Mirage::MOUSEEVENT_PRESS, MODIFIER_NONE |
                             ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                             ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));

  CView::OnRButtonDown(nFlags, point);
}

void ModelView::OnRButtonUp(UINT nFlags, CPoint point) 
{
  double view_x, view_y;

	if (GetCapture()) {
		ReleaseCapture();

    windowspace_to_viewspace(point, &view_x, &view_y);

    modeller_view->mouse_event(view_x, view_y,
                               2, Mirage::MOUSEEVENT_RELEASE, MODIFIER_NONE |
                               ((nFlags & MK_CONTROL) ? MODIFIER_CTRL  : 0) |
                               ((nFlags & MK_SHIFT)   ? MODIFIER_SHIFT : 0));
  }
	CView::OnRButtonUp(nFlags, point);
}

void ModelView::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo) 
{
  PrepareForRender();

	CView::OnPrepareDC(pDC, pInfo);
}

void ModelView::OnDestroy() 
{
	if (m_pDC)         delete m_pDC;
  if (modeller_view) delete modeller_view;
  if (bAPICreated)   ReleaseAPI();

  m_pDC = NULL;
  bAPICreated = FALSE;
  modeller_view = NULL;

	CView::OnDestroy();
}

BOOL ModelView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
  modeller_view->zoom_event(zDelta / 120);
    
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

//////////////////////////////////////////////////////////////////////
// Virtual function stubs... overridden by specific API ModelViews.

// Set up the 3D API.
HRESULT ModelView::InitAPI(void)
{
  return S_OK;
}

// Create the Modeller's view class to interface to.
Mirage::View *ModelView::CreateModellerView(void)
{
  return NULL;
}

// Ensure that the right context is selected for rendering.
void ModelView::PrepareForRender(void)
{
}

// Release the 3D API.
void ModelView::ReleaseAPI(void)
{
}
