/*
 * trimem.h: Definitions for class TriMemory.
 *
 * Contains code to construct a TriMemory which is a variant of the Mirage-style
 * blocklist that's been specialised to handle strings of triangles.
 */

// The default values for tri_memblocks and blocksize that
// are used if no values are specified in the constructor.
const int trimem_default_blocksize = 1000;
const int trimem_default_maxblocks = 1000;

// Indices into the adjacency array... used for collectionspace
// adjacency determination.
#define ADJ_LEN    2
#define ADJ_INDEX  0
#define ADJ_VERTEX 1

typedef struct tri_ {
  // The length of the string that this triangle is part
  // of. This is -1 for deleted triangles and 0 for newly
  // allocated triangles.
  int string_length;

  // Pointer to the next triangle in the string. If a deleted
  // or unallocated triangle, points to the next free triangle
  // in the TriMemory.
  struct tri_ *stringptr;

  // Triangle vertices
  vector v[3];

  // Triangle normals (for Gouraud shading) and lighting values.
  vector n[3];

  // Which edges need to be highlighted?
  unsigned char highlight_edge;

  // Adjacency information for this triangle (relative
  // within the celll).
  int adj[3][ADJ_LEN];
} tri;

class TriMemory {
private:
  int max_memblocks, blocksize, blocks_used;
  tri *next_free, **storage;

public:
  // Constructor: allows specified blocksize and maxblocks.
  TriMemory(int blocksize = trimem_default_blocksize,
            int max_memblocks = trimem_default_maxblocks);

  // Destructor; deallocate all memory and exit.
  ~TriMemory(void);

  // Allocate a new triangle from the storage and return a pointer to it.
  tri *allocate(void);

  // Deallocate a previously allocated triangle.
  void deallocate(tri *the_tri);

  // Deallocate a string of triangles.
  void deallocate_string(tri *the_tri);
};
