/*
 * copytracker.cc: Rountines for class CopyTracker and supporting code.
 *
 * Contains code to create and maintain the CopyTracker and administer the
 * copies that are maintained of an objectnode.
 */

/* Collection-safe */

// #include "includelist.h"
#include "Stdafx.h"

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

/*
 *  Routines for CopyTracker
 */

// Select the copies of an object.
void CopyTracker::select_copies(void)
{
  for (Tracker *ptr = trackerlist_head; ptr != NULL; ptr = ptr->next)
    ptr->copy->select();
}

// Deselect the copies of an object.
void CopyTracker::deselect_copies(void)
{
  for (Tracker *ptr = trackerlist_head; ptr != NULL; ptr = ptr->next)
    ptr->copy->deselect();
}

// Add a new Tracker to the CopyTracker.
Tracker *CopyTracker::add_new_tracker(primtreenode *new_primnode)
{
  Tracker *new_tracker = new Tracker(this, new_primnode);

  // Insert new_tracker into the doubly-linked list.
  new_tracker->timestamp = timestamp;
  new_tracker->prev = trackerlist_tail;
  new_tracker->next = NULL;
  
  if (new_tracker->prev == NULL)
    trackerlist_head = new_tracker;
  else
    new_tracker->prev->next = new_tracker;
  
  trackerlist_tail = new_tracker;
  num_copies++;

  return new_tracker;
}

// Remove a Tracker from the CopyTracker.
void CopyTracker::remove_tracker(Tracker *killme)
{
  assert(num_copies > 0);

  if (killme->prev == NULL)
    trackerlist_head = killme->next;
  else
    killme->prev->next = killme->next;

  if (killme->next == NULL)
    trackerlist_tail = killme->prev;
  else
    killme->next->prev = killme->prev;

  num_copies--;

  delete killme;
}

// Advance the timestamp variable.
void CopyTracker::advance_timestamp(void)
{
  timestamp++;
}

// Compare the CopyTracker's timestamp to that of a Tracker. If this
// returns TRUE, the Tracker has been added since the timestamp change; if
// this returns FALSE, the Tracker is an old one, pre-timestamp-change.
int CopyTracker::has_current_timestamp(Tracker *tracker)
{
  return (tracker->timestamp == timestamp);
}

// Called when a primtreenode is deleted, gets rid of its tracker.
// Returns TRUE if its objectnode has no primtreenode equivalents left,
// and needs to be deleted.
int CopyTracker::notify_primtreenode_deletion(Tracker *tracker)
{
  assert(tracker != NULL);

  // Remove the Tracker from the CopyTracker.
  remove_tracker(tracker);

  // If the CopyTracker is empty, the objectnode serves no purpose now
  // and must be deleted.
  if (trackerlist_head == NULL) {
    assert(trackerlist_tail == NULL);
    return TRUE;
  } else {
    return FALSE;
  }
}


#ifdef USE_NAMESPACES
} // namespace Mirage {
#endif
