/*
 * graphview.h: Definitions for GraphView and its subclasses.
 *
 * Contains code to draw the graph that has been laid out by the GraphLayout.
 */

/* Collection-safe */

typedef enum {VS_SELECT, VS_DELETE, VS_LINK_PLUS, VS_LINK_MINUS, VS_LINK_AND} graphviewstate;

// GraphView is subclassed for each platform, and contains OS-specific
// drawing calls.
class GraphView {
public:
  Modeller *modeller;

  // The state of the graphview.
  graphviewstate viewstate;

  // Routine called when a node is selected in the graphview.
  void select_node(objectnode *node);

  // Set the state of the graphview.
  virtual void setgraphviewstate(graphviewstate viewstate) = 0;

  // Clear the canvas.
  virtual void clear_canvas(void) = 0;

  // Add a node to the canvas. If this is selectable, then it must have
  // a non-NULL selection_node. NB: the linktype indicators use this, too.
  virtual void add_node(int xpos, int ypos, int size, int selected,
			char *internal_text, char *external_text,
			objectnode *selection_node) = 0;

  // Add a link to the canvas.
  virtual void add_link(int num_segs, int *seg_coord,
			int is_ghost, int has_arrow) = 0;

  // Called by the layout engine when the drawing pass is completed.
  virtual void finish_drawing(void) = 0;
};
