/*
 * parenttracker.cc: Routines for class ParentTracker.
 *
 * Contains code to create and maintain a ParentTracker, which tracks the
 * number of links that are made to an objectnode in the CSGgraph.
 * NB: ParentTracker tracks groupobjectnode->group and
 * repeatobjectnode->repeat links, but not objectnode->linkptr.
 */

/* Collection-safe */

// #include "includelist.h"
#include "Stdafx.h"

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

/*
 *  Routines for ParentTracker
 */

// Add a new parent.
void ParentTracker::add(objectnode *new_parent)
{
  parenttrackernode *new_node;

  new_node = (parenttrackernode *) malloc(sizeof(parenttrackernode));
  if (!new_node)
    fatal_error("ParentTracker::add", "Can't allocate new node!",
		"Out of memory.");

  new_node->owner = this;
  new_node->parent = new_parent;

  // Insert new_node into the doubly-linked list, at the end.
  new_node->prev = tail;
  new_node->next = NULL;

  if (new_node->prev == NULL)
    head = new_node;
  else
    new_node->prev->next = new_node;
  
  tail = new_node;
  num_links++;
}

// Remove a Tracker from the ParentTracker.
void ParentTracker::remove(objectnode *parent)
{
  parenttrackernode *ptr;

  for (ptr = head; ptr != NULL; ptr = ptr->next)
    if (ptr->parent == parent)
      break;

  assert(ptr != NULL);

  if (ptr->prev == NULL)
    head = ptr->next;
  else
    ptr->prev->next = ptr->next;

  if (ptr->next == NULL)
    tail = ptr->prev;
  else
    ptr->next->prev = ptr->prev;

  free(ptr);

  assert(num_links > 0);
  num_links--;
}

// Print out the ParentTracker.
void ParentTracker::print(int indent)
{
  parenttrackernode *ptr;

  printf("%*sparents: ", indent + 1, "");
  for (ptr = head; ptr != NULL; ptr = ptr->next) {
    printf("C%03d", ptr->parent->index);
    if (ptr->next != NULL)
      printf(", ");
  }
  printf("\n");
}
  

#ifdef USE_NAMESPACES
} // namespace Mirage {
#endif
