/*
 * parenttracker.h: Definitions for class ParentTracker and supporting code.
 *
 * Contains code to create and maintain a ParentTracker, which tracks the
 * number of links that are made to an objectnode in the CSGgraph.
 * NB: ParentTracker tracks groupobjectnode->group and
 * repeatobjectnode->repeat links, but not objectnode->linkptr.
 */

/* Collection-safe */

typedef struct parenttrackernode_ {
  // The ParentTracker that this is attached to.
  ParentTracker *owner;

  // The objectnode that is the parent.
  objectnode *parent;

  // Pointers to the next and previous nodes in the linked list.
  struct parenttrackernode_ *prev, *next;
} parenttrackernode;

class ParentTracker {
public:
  // The objectnode whose parents this tracks.
  objectnode *owner;

  // Total number of links to this object, and a linked list that holds
  // parenttrackernodes.
  int num_links;
  parenttrackernode *head, *tail;

  // Register a new parent.
  void add(objectnode *parent);

  // Remove a parent.
  void remove(objectnode *parent);

  // Print out the ParentTracker.
  void print(int indent);
  
  // Construct a new, empty ParentTracker.
  ParentTracker(objectnode *owner) {
    this->owner = owner;
    num_links = 0;
    head = tail = NULL;
  }

  // Delete the ParentTracker.
  ~ParentTracker(void)
  {
    parenttrackernode *current, *last;

    for (current = head; current != NULL ;) {
      last = current;
      current = last->next;
      free(last);
    }
  }      
};
