/*
 * stringmem.h: Definitions for class StringMemory.
 *
 * Contains code to construct a StringMemory which is a variant of the Mirage-style
 * blocklist that's been specialised to handle cells of strings.
 */

// The default values for string_memblocks and blocksize that
// are used if no values are specified in the constructor.
const int stringmem_default_blocksize = 1000;
const int stringmem_default_maxblocks = 1000;

typedef struct string_ {
  // The number of strings in this celllist. This is -1 for deleted strings and 0 for newly
  // allocated strings.
  int celllist_length;

  // The number of references to the string. This is -1 for all
  // strings that aren't celllist heads. References are only counted
  // if they are from a Collection / WorldCollection.
  int reference_count;

  // The number of triangles in the celllist (or fraction thereof)
  int num_tri;

  // Normal vector and distance from plane-through-origin for the string.
  vector norm;
  double plane_dot;

  // Pointer to the next string in the celllist. If a deleted
  // or unallocated string, points to the next free string
  // in the StringMemory.
  struct string_ *stringptr;

  // The first triangle in our string.
  tri *head;
} string;

class StringMemory {
private:
  Modeller *owner;
  int max_memblocks, blocksize, blocks_used;
  string *next_free, **storage;

public:
  // Constructor: allows specified blocksize and maxblocks.
  StringMemory(Modeller *owner, int blocksize = stringmem_default_blocksize,
            int max_memblocks = stringmem_default_maxblocks);

  // Destructor; deallocate all memory and exit.
  ~StringMemory(void);

  // Allocate a new triangle from the storage and return a pointer to it.
  string *allocate(void);

  // Deallocate a previously allocated triangle.
  void deallocate(string *the_string);

  // Deallocate a celllist of strings.
  void deallocate_celllist(string *the_string);
};
