/*
 * transformlist.h: Definitions for class TransformList and supporting code.
 *
 * Contains code to create and maintain a set of transformations that are
 * being applied to an object.
 */

// A node in the linked list of transformations.
typedef struct TransformListNode_ {
  ObjectTransformation *trans;
  transform matrix, cumulative;

  struct TransformListNode_ *prev, *next;
} TransformListNode;

class TransformList {
public:
  int num_transforms;
  TransformListNode *head, *tail;
  
  // Get the total transformation.
  void get_position(transform t);

  // Update the transformation matrix at a node.
  void update_transform(TransformListNode *n);

  // Propagate the transformation matrices, starting from a node.
  void propagate_matrices(void);
  void propagate_matrices(TransformListNode *n);

  // Insert a node into the list. Used internally by add_transformation and
  // move_transformation.
  void insert_node(TransformListNode *n, TransformListNode *pos);

  // Remove a node from the list. Used internally by delete_transformation and
  // move_transformation.
  void remove_node(TransformListNode *n);

  // Add a transformation to the list, immediately after the given node.
  // NULL means at the head.
  void add_transformation(ObjectTransformation *objtrans,
			  TransformListNode *n);

  // Move a transformation to a new position in the list, immediately after
  // new_pos. NULL means at the head.
  void move_transformation(TransformListNode *n, TransformListNode *new_pos);

  // Delete a transformation.
  void delete_transformation(TransformListNode *n);

  // Print out the list.
  void print(void);

  // Create a new, empty, TransformList.
  TransformList(void) {
    num_transforms = 0;
    head = tail = NULL;
  }

  // Create a non-empty TransformList.
  TransformList(int num_trans, ObjectTransformation **trans_array);

  // Delete the TransformList.
  ~TransformList(void);
};
