/*
 * wgraphview.h: Definitions for WObjectGraphView.
 *
 * Contains routines to draw the graph that has been laid out by the
 * GraphLayout, within a Windows view.
 */

/* Collection-safe */

#ifdef _WIN32

#ifdef USE_NAMESPACES
namespace Mirage {
#endif

const int MAX_OBJECTGRAPH_LABELSIZE = 20;

typedef struct nodeinfo_ {
  int pos[2];
  int size;
  int is_selected;
  char internaltext[MAX_OBJECTGRAPH_LABELSIZE];
  char externaltext[MAX_OBJECTGRAPH_LABELSIZE];
  objectnode *objectgraphptr;
} nodeinfo;

const int MAX_OBJECTGRAPH_LINKSEGS = 10;

typedef struct linkinfo_ {
  int num_segs;
  int segpos[MAX_OBJECTGRAPH_LINKSEGS][2];
  int is_ghost;
  int has_arrow;
} linkinfo;

// WObjectGraphView is attached to a Windows CGraphView and displays nodes in it.
class WObjectGraphView : public GraphView {
public:
  // The Windows class to call.
  class CGraphView *graphview;

  // These member variables hold the current state of the objectgraphview.
  // They contain data about the position of each node, link, etc.
  int num_nodes, nodearray_size, nodearray_blocksize;
  nodeinfo *nodearray;
  int num_links, linkarray_size, linkarray_blocksize;
  linkinfo *linkarray;

  // Get a new pointer into nodearray or linkarray.
  nodeinfo *new_nodeinfo(void);
  linkinfo *new_linkinfo(void);

  // Find the size of the objects on the canvas.
  void get_size(int *xsize, int *ysize);

  // The user has clicked on the canvas to select a node.
  void selection_click(int x, int y);
  
  // Set the state of the graphview.
  virtual void setgraphviewstate(graphviewstate viewstate);

  // Clear the canvas.
  virtual void clear_canvas(void);

  // Add a node to the canvas. If this is selectable, then it must have
  // a non-NULL selection_node. NB: the linktype indicators use this, too.
  virtual void add_node(int xpos, int ypos, int size, int selected,
			char *internal_text = NULL, char *external_text = NULL,
			objectnode *selection_node = NULL);

  // Add a link to the canvas.
  virtual void add_link(int num_segs, int *seg_coord,
			int is_ghost, int has_arrow);

  // Called by the layout engine when the drawing pass is completed.
  // Tells us to update our view.
  virtual void finish_drawing(void);

  // Create (and bind) the view.
  WObjectGraphView(Modeller *m, CGraphView *graphview);

  // Delete (and unbind) the view.
  ~WObjectGraphView();
};

#ifdef USE_NAMESPACES
} // namespace Mirage {
#endif

#endif  // #ifdef _WIN32

