/*
 * worldstringmem.h: Definitions for class WorldStringMemory.
 *
 * Contains code to construct a WorldStringMemory which is a variant of the Mirage-style
 * blocklist that's been specialised to handle cells of worldstrings.
 */

// The default values for worldstring_memblocks and blocksize that
// are used if no values are specified in the constructor.
const int worldstringmem_default_blocksize = 1000;
const int worldstringmem_default_maxblocks = 1000;

typedef struct worldstring_ {
  // The number of worldstrings in this celllist. This is -1 for deleted worldstrings and 0 for newly
  // allocated worldstrings.
  int celllist_length;

  // The number of references to the worldstring. This is -1 for all
  // worldstrings that aren't celllist heads. References are only counted
  // if they are from a Collection / WorldCollection.
  int reference_count;

  // Worldspace normal vector and plane dot product for the string.
  vector norm;
  double plane_dot;

  // Pointer to the next worldstring in the celllist. If a deleted
  // or unallocated worldstring, points to the next free worldstring
  // in the WorldStringMemory.
  struct worldstring_ *stringptr;
} worldstring;

class WorldStringMemory {
private:
  Modeller *owner;
  int max_memblocks, blocksize, blocks_used;
  worldstring *next_free, **storage;

public:
  // Constructor: allows specified blocksize and maxblocks.
  WorldStringMemory(Modeller *owner, int blocksize = worldstringmem_default_blocksize,
            int max_memblocks = worldstringmem_default_maxblocks);

  // Destructor; deallocate all memory and exit.
  ~WorldStringMemory(void);

  // Allocate a new triangle from the storage and return a pointer to it.
  worldstring *allocate(void);

  // Deallocate a previously allocated triangle.
  void deallocate(worldstring *the_worldstring);

  // Deallocate a celllist of worldstrings.
  void deallocate_celllist(worldstring *the_worldstring);
};
