// CTransView.cpp : implementation file
//

#include "stdafx.h"
#include "ModelApp.h"
#include "ModelDoc.h"
#include "CTransView.h"
#include "wtransview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTransView

IMPLEMENT_DYNCREATE(CTransView, CScrollView)

CTransView::CTransView()
{
  wtransview = NULL;
  lineskip = 10;

  // Remainder of initialisation done in Init().
}

CTransView::~CTransView()
{
  // Most deletion is done in OnDestroy().
}


BEGIN_MESSAGE_MAP(CTransView, CScrollView)
	//{{AFX_MSG_MAP(CTransView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTransView drawing

void CTransView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
  TEXTMETRIC tm;
  int i, max_paramlength;
  CSize textsize;

  pDC->GetTextMetrics((LPTEXTMETRIC) &tm);
  lineskip = tm.tmHeight + 1;

  max_paramlength = 0;

  for (i = 0; i < wtransview->num_trans; i++) {
    textsize = pDC->GetTextExtent((LPCTSTR) wtransview->paramstring[i], strlen(wtransview->paramstring[i]));
    if (textsize.cx > max_paramlength)
      max_paramlength = textsize.cx;
  }
  
  // Draw the transformation information.
  for (i = 0; i < wtransview->num_trans; i++) {
    if (i == wtransview->selected_trans) {
      pDC->FillSolidRect(CRect(0, lineskip*i, 500, lineskip*(i+1)), RGB(0, 0, 150));
      pDC->SetTextColor(RGB(255, 255, 255));
    }

    pDC->TextOut(0,  lineskip*i, (LPCTSTR) wtransview->namestring[i],  strlen(wtransview->namestring[i]));
    pDC->TextOut(80, lineskip*i, (LPCTSTR) wtransview->paramstring[i], strlen(wtransview->paramstring[i]));

    if (i == wtransview->selected_trans) {
      pDC->SetBkColor(RGB(255, 255, 255));
      pDC->SetTextColor(RGB(0, 0, 0));
    }
  }

  // Draw the 'move-to' line.
  if ((wtransview->movetoline != -1) &&
      (wtransview->movetoline != wtransview->selected_trans) &&
      (wtransview->movetoline != wtransview->selected_trans + 1)) {
    CPen *oldPen;

    CPen newPen(PS_SOLID, 1, RGB(0, 0, 150));

    oldPen = pDC->SelectObject(&newPen);

    pDC->MoveTo(0,   lineskip * wtransview->movetoline);
    pDC->LineTo(500, lineskip * wtransview->movetoline);

    pDC->SelectObject(oldPen);
  }

  // Set the scrollable size.
  SetScrollSizes(MM_TEXT, CSize(80 + max_paramlength, wtransview->num_trans * lineskip));
}

/////////////////////////////////////////////////////////////////////////////
// CTransView diagnostics

#ifdef _DEBUG
void CTransView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CTransView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CModelDoc* CTransView::GetDocument() // non-debug version is inline and in .h
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CModelDoc)));
	return (CModelDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTransView message handlers

int CTransView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CScrollView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create the Mirage WTransView class that's attached to this View.
  wtransview = new Mirage::WTransView(GetDocument()->modeller, this);

  // Set the initial size of the graphview.
  SetScrollSizes(MM_TEXT, CSize(50, 50));

  return 0;
}

void CTransView::OnDestroy() 
{
	CScrollView::OnDestroy();
	
  // Delete the WTransView.
  if (wtransview != NULL) {
    delete wtransview;
    wtransview = NULL;
  }
}


void CTransView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
  int new_selection;
  Mirage::SkitterList *skitterlist = wtransview->modeller->skitterlist;
  Mirage::Skitter *sk;
  Mirage::ObjectTransformation *selectedtrans;
  
  // We've registered a double-click in the transformation view.

 	// CScrollView changes the viewport origin and mapping
	// mode. Thus, we must convert the mouse-point from
	// device coordinates to logical coordinates relative
	// to the document.
	CClientDC dc(this);
	OnPrepareDC(&dc); // Set up origin
	dc.DPtoLP(&point); // Move point to logical coords.

  // Which transformation is this that's being selected?
  new_selection = point.y / lineskip;

  if ((new_selection >= 0) && (new_selection < wtransview->num_trans)) {
    // First, select this skitter.
    sk = skitterlist->copies[0].skitter[(wtransview->num_trans - 1) - new_selection];
    selectedtrans = sk->transformation->trans;

    skitterlist->dmd_select(sk);
    wtransview->modeller->update_all_views();

    // Now, pop up a dialog box to handle this.
    TRACE("dialog box popup for %s\n", wtransview->namestring[new_selection]);

    Mirage::vector new_trans;

    // FIXME: nasty debugging hack
    Mirage::Vector::set(new_trans, ((Mirage::ObjectTranslation *) skitterlist->current_dmd->transformation->trans)->trans_amount);
    new_trans[2] += 0.001;
    ((Mirage::ObjectTranslation *) skitterlist->current_dmd->transformation->trans)->set_translation(new_trans);


    // Update the changes in the TransformList.
    skitterlist->selected_node->transformlist->update_transform(skitterlist->current_dmd->transformation);
    skitterlist->selected_node->transformlist->propagate_matrices();
    skitterlist->update_all_skitters();
    wtransview->modeller->update_all_transviews();
    wtransview->modeller->csg_graph->update_transformed_object(skitterlist->selected_node);
    wtransview->modeller->update_all_views();
  }
}

void CTransView::OnLButtonDown(UINT nFlags, CPoint point) 
{
  int new_selection;
  Mirage::SkitterList *skitterlist = wtransview->modeller->skitterlist;
  
  // We've registered a left-button press in the transformation view.
  // Capture the mouse (used for drag ops).
	SetCapture();

 	// CScrollView changes the viewport origin and mapping
	// mode. Thus, we must convert the mouse-point from
	// device coordinates to logical coordinates relative
	// to the document.
	CClientDC dc(this);
	OnPrepareDC(&dc); // Set up origin
	dc.DPtoLP(&point); // Move point to logical coords.

  // Which transformation is this that's being selected?
  new_selection = point.y / lineskip;

  if ((new_selection < 0) || (new_selection >= wtransview->num_trans)) {
    // Deselect all transformations.
    skitterlist->dmd_deselect();
  } else {
    // Select a particular transformation.
    skitterlist->dmd_select(skitterlist->copies[0].skitter[(wtransview->num_trans - 1) - new_selection]);
  }
  wtransview->modeller->update_all_views();
}

void CTransView::OnLButtonUp(UINT nFlags, CPoint point) 
{
  // Release the capture, if we have one.
  if (GetCapture())
    ReleaseCapture();

  wtransview->check_move();
}

void CTransView::OnMouseMove(UINT nFlags, CPoint point) 
{
  int new_pos;

  if (GetCapture() == this) {
    // We are in the middle of a drag operation.

   	// CScrollView changes the viewport origin and mapping
  	// mode. Thus, we must convert the mouse-point from
  	// device coordinates to logical coordinates relative
  	// to the document.
  	CClientDC dc(this);
	  OnPrepareDC(&dc); // Set up origin
  	dc.DPtoLP(&point); // Move point to logical coords.
    
    // Calculate the nearest position that we could
    // be moving to.
    new_pos = (point.y + (lineskip / 2)) / lineskip;

    if (new_pos < 0)
      new_pos = 0;
    else if (new_pos > wtransview->num_trans)
      new_pos = wtransview->num_trans;

    // Set the position of the selection cursor.
    if (new_pos != wtransview->movetoline) {
      wtransview->movetoline = new_pos;
      Invalidate();
    }
  }
}

void CTransView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
  // Catch the DEL key and delete a transformation. Only
  // if bit 14 of nFlags is zero, indicating that this is
  // the first down transition on a keypress (i.e. not a
  // repeat)
  if ((nChar == 0x2E) && ((nFlags & (1 << 14)) == 0)) {
    wtransview->delete_trans();
  }
  
	CScrollView::OnKeyDown(nChar, nRepCnt, nFlags);
}
