/*
 * transview.h: Definitions for TransView.
 *
 * Contains code to interface with the platform-specific transformation view.
 */

#define MAX_TRANSVIEW_SIZE   30
#define MAX_TRANSVIEW_STRLEN 80

// TransView is subclassed for each platform, and contains OS-specific
// interface calls.
class TransView {
public:
  Modeller *modeller;

  // The number of transformations in the window.
  int num_trans;

  // The strings that are displayed for each transformation.
  char namestring [MAX_TRANSVIEW_SIZE][MAX_TRANSVIEW_STRLEN];
  char paramstring[MAX_TRANSVIEW_SIZE][MAX_TRANSVIEW_STRLEN];

  // The selected transformation.
  int selected_trans;

  // Position of the move-to line (if there is one drawn).
  int movetoline;

  // Redraw the transview window.
  virtual void redraw(void) = 0;

  // Delete the currently selected transformation.
  void delete_trans(void);

  // Check to see if a move operation needs to
  // be performed (after a mouse-up).
  void check_move(void);

  // Add a new transformation.
  void add_trans(trans_type ttype);
  void add_trans(ObjectTransformation *new_trans);

  // Update the transformation view so that the correct
  // transformation (if any) is selected.
  void update(void);
};
